
exception ConversionError of string

type uri = string
type role =
  | ROLE of uri
  | INVERSE of role

type class_exp =
  | THING
  | NOTHING
  | NAMED   of uri
  | NOT     of class_exp
  | AND     of class_exp list
  | OR      of class_exp list
  | SOME    of role * class_exp
  | ALL     of role * class_exp
  | MIN     of int * role * class_exp
  | MAX     of int * role * class_exp
  (* TODO: | OneOf of  * formula *)


type declaration =
  | CLASS of uri
  | DATATYPE of uri
  | OBJECTPROPERTY of uri
  | DATAPROPERTY of uri
  | ANNOTATIONPROPERTY of uri
  | NAMEDINDIVIDUAL of uri

type axiom =
  | ANNOTATION of string
  (* class expression axioms *)
  | DECLARATION of declaration
  | SUBCLASS of class_exp * class_exp
  | EQUIVALENTCLASSES of class_exp list
  | DISJOINTCLASSES of class_exp list
  | DISJOINTUNION   of class_exp * (class_exp list)

type prefix = string * string (* prefix name to long substitution *)
type prefixes = prefix list
type ontology = string * (axiom list) (* ontology uri and axioms *)

val string_of_prefix : prefix -> string
val string_of_ontology : ontology -> string
val string_of_axiom : axiom -> string
val string_of_class_exp : class_exp -> string

(* try to convert some OWL ontology to coalg formulas *)
(* first try to express it in pure K, then in GML *)
val to_coalg : ontology -> CoAlgReasoner.sortTable * CoAlgReasoner.nomTable * CoAlgReasoner.assumptions

val coalg_formula_of_owl : class_exp -> CoAlgFormula.formula
val coalg_global_assumption_of_owl : axiom -> CoAlgFormula.formula

(* vim: set et sw=2 sts=2 ts=8 : *)
